@@ -27,6 +27,7 @@
 static PyObject* CBSONError;
 static PyObject* InvalidName;
 static PyObject* InvalidDocument;
+static PyObject* InvalidStringData;
 static PyObject* SON;
 static PyObject* Binary;
 static PyObject* Code;
@@ -45,8 +46,170 @@ typedef int Py_ssize_t;
 
 /* TODO we ought to check that the malloc or asprintf was successful
  * and raise an exception if not. */
-#ifdef _MSC_VER
-#define GMTIME_INVERSE _mkgmtime64
+#if defined(WIN32) || defined(_MSC_VER)
+
+/* no mkgmtime on MSVC before VS 2005. this is terribly gross */
+#if defined(_MSC_VER) && (_MSC_VER >= 1400)
+#define GMTIME_INVERSE(time_struct) _mkgmtime64(time_struct)
+#else
+/* Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ *
+ *
+ */
+/*
+ * Copyright (c) 1987, 1989, 1993
+ *    The Regents of the University of California.  All rights reserved.
+ *
+ * This code is derived from software contributed to Berkeley by
+ * Arthur David Olson of the National Cancer Institute.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. All advertising materials mentioning features or use of this software
+ *    must display the following acknowledgement:
+ *    This product includes software developed by the University of
+ *    California, Berkeley and its contributors.
+ * 4. Neither the name of the University nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
+ */
+
+/*
+** Adapted from code provided by Robert Elz, who writes:
+**    The "best" way to do mktime I think is based on an idea of Bob
+**    Kridle's (so its said...) from a long time ago. (mtxinu!kridle now).
+**    It does a binary search of the time_t space.  Since time_t's are
+**    just 32 bits, its a max of 32 iterations (even at 64 bits it
+**    would still be very reasonable).
+*/
+#ifndef WRONG
+#define WRONG     (-1)
+#endif /* !defined WRONG */
+
+static int tmcomp(atmp, btmp)
+     register const struct tm * const atmp;
+     register const struct tm * const btmp;
+{
+    register int      result;
+
+    if ((result = (atmp->tm_year - btmp->tm_year)) == 0 &&
+        (result = (atmp->tm_mon - btmp->tm_mon)) == 0 &&
+        (result = (atmp->tm_mday - btmp->tm_mday)) == 0 &&
+        (result = (atmp->tm_hour - btmp->tm_hour)) == 0 &&
+        (result = (atmp->tm_min - btmp->tm_min)) == 0)
+        result = atmp->tm_sec - btmp->tm_sec;
+    return result;
+}
+
+time_t mkgmtime(tmp)
+struct tm * const tmp;
+{
+    register int                  dir;
+    register int                  bits;
+    register int                  saved_seconds;
+    time_t                        t;
+    struct tm               yourtm, *mytm;
+
+    yourtm = *tmp;
+    saved_seconds = yourtm.tm_sec;
+    yourtm.tm_sec = 0;
+
+    /*
+    ** Calculate the number of magnitude bits in a time_t
+    ** (this works regardless of whether time_t is
+    ** signed or unsigned, though lint complains if unsigned).
+    */
+    for (bits = 0, t = 1; t > 0; ++bits, t <<= 1)
+        ;
+    /*
+    ** If time_t is signed, then 0 is the median value,
+    ** if time_t is unsigned, then 1 << bits is median.
+    */
+    t = (t < 0) ? 0 : ((time_t) 1 << bits);
+
+    /* Some gmtime() implementations are broken and will return
+     * NULL for time_ts larger than 40 bits even on 64-bit platforms
+     * so we'll just cap it at 40 bits */
+    if(bits > 40) bits = 40;
+
+    for ( ; ; ) {
+        mytm = gmtime(&t);
+
+        if(!mytm) return WRONG;
+
+        dir = tmcomp(mytm, &yourtm);
+        if (dir != 0) {
+            if (bits-- < 0)
+                return WRONG;
+            if (bits < 0)
+                --t;
+            else if (dir > 0)
+                t -= (time_t) 1 << bits;
+            else  t += (time_t) 1 << bits;
+            continue;
+        }
+        break;
+    }
+    t += saved_seconds;
+    return t;
+}
+#define GMTIME_INVERSE(time_struct) mkgmtime(time_struct)
+#endif
+
 #define INT2STRING(buffer, i)                   \
     {                                           \
         int vslength = _scprintf("%d", i) + 1;  \
@@ -54,7 +217,7 @@ typedef int Py_ssize_t;
         _snprintf(*buffer, vslength, "%d", i);  \
     }
 #else
-#define GMTIME_INVERSE timegm
+#define GMTIME_INVERSE(time_struct) timegm(time_struct)
 #define INT2STRING(buffer, i) asprintf(buffer, "%d", i);
 #endif
 
@@ -153,7 +316,7 @@ static int write_string(bson_buffer* buffer, PyObject* py_string) {
 
     for (i = 0; i < string_length - 1; i++) {
         if (string[i] == 0) {
-            PyErr_SetString(InvalidDocument, "BSON strings must not contain a NULL character");
+            PyErr_SetString(InvalidStringData, "BSON strings must not contain a NULL character");
             return 0;
         }
     }
@@ -167,6 +330,17 @@ static int write_string(bson_buffer* buffer, PyObject* py_string) {
     return 1;
 }
 
+/* returns 0 on invalid ascii */
+static int validate_ascii(const char* data, int length) {
+    int i;
+    for (i = 0; i < length; i++) {
+        if (data[i] & 0x80) {
+            return 0;
+        }
+    }
+    return 1;
+}
+
 /* TODO our platform better be little-endian w/ 4-byte ints! */
 /* returns 0 on failure */
 static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject* value, unsigned char check_keys) {
@@ -205,7 +379,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
     } else if (PyDict_Check(value)) {
         *(buffer->buffer + type_byte) = 0x03;
         return write_dict(buffer, value, check_keys);
-    } else if (PyList_CheckExact(value)) {
+    } else if (PyList_Check(value) || PyTuple_Check(value)) {
         int start_position,
             length_location,
             items,
@@ -222,7 +396,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
             return 0;
         }
 
-        items = PyList_Size(value);
+        items = PySequence_Size(value);
         for(i = 0; i < items; i++) {
             int list_type_byte = buffer_save_bytes(buffer, 1);
             char* name;
@@ -242,10 +416,12 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
             }
             free(name);
 
-            item_value = PyList_GetItem(value, i);
+            item_value = PySequence_GetItem(value, i);
             if (!write_element_to_buffer(buffer, list_type_byte, item_value, check_keys)) {
+                Py_DECREF(item_value);
                 return 0;
             }
+            Py_DECREF(item_value);
         }
 
         /* write null byte and fill in length */
@@ -330,17 +506,14 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         memcpy(buffer->buffer + length_location, &length, 4);
         return 1;
     } else if (PyString_Check(value)) {
-        PyObject* encoded;
         int result;
 
         *(buffer->buffer + type_byte) = 0x02;
-        /* we have to do the encoding so we can fail fast if they give us non utf-8 */
-        encoded = PyString_AsEncodedObject(value, "utf-8", "strict");
-        if (!encoded) {
+        if (!validate_ascii(PyString_AsString(value), PyString_Size(value))) {
+            PyErr_SetString(InvalidStringData, "strings in documents must be ASCII only");
             return 0;
         }
-        result = write_string(buffer, encoded);
-        Py_DECREF(encoded);
+        result = write_string(buffer, value);
         return result;
     } else if (PyUnicode_Check(value)) {
         PyObject* encoded;
@@ -374,7 +547,7 @@ static int write_element_to_buffer(bson_buffer* buffer, int type_byte, PyObject*
         *(buffer->buffer + type_byte) = 0x09;
         return buffer_write_bytes(buffer, (const char*)&time_since_epoch, 8);
     } else if (PyObject_IsInstance(value, ObjectId)) {
-        PyObject* pystring = PyObject_Str(value);
+        PyObject* pystring = PyObject_GetAttrString(value, "_ObjectId__id");
         if (!pystring) {
             return 0;
         }
@@ -818,9 +991,8 @@ static PyObject* get_value(const char* buffer, int* position, int type) {
     case 13:
     case 14:
         {
-            int value_length;
+            int value_length = ((int*)(buffer + *position))[0] - 1;
             *position += 4;
-            value_length = strlen(buffer + *position);
             value = PyUnicode_DecodeUTF8(buffer + *position, value_length, "strict");
             if (!value) {
                 return NULL;
@@ -1155,11 +1327,48 @@ static PyObject* _cbson_bson_to_dict(PyObject* self, PyObject* bson) {
     return result;
 }
 
+static PyObject* _cbson_to_dicts(PyObject* self, PyObject* bson) {
+    int size;
+    Py_ssize_t total_size;
+    const char* string;
+    PyObject* dict;
+    PyObject* result;
+
+    if (!PyString_Check(bson)) {
+        PyErr_SetString(PyExc_TypeError, "argument to _to_dicts must be a string");
+        return NULL;
+    }
+    total_size = PyString_Size(bson);
+    string = PyString_AsString(bson);
+    if (!string) {
+        return NULL;
+    }
+
+    result = PyList_New(0);
+
+    while (total_size > 0) {
+        memcpy(&size, string, 4);
+
+        dict = elements_to_dict(string + 4, size - 5);
+        if (!dict) {
+            return NULL;
+        }
+        PyList_Append(result, dict);
+        Py_DECREF(dict);
+        string += size;
+        total_size -= size;
+    }
+
+    return result;
+}
+
 static PyMethodDef _CBSONMethods[] = {
     {"_dict_to_bson", _cbson_dict_to_bson, METH_VARARGS,
      "convert a dictionary to a string containing it's BSON representation."},
     {"_bson_to_dict", _cbson_bson_to_dict, METH_O,
      "convert a BSON string to a SON object."},
+    {"_to_dicts", _cbson_to_dicts, METH_O,
+     "convert binary data to a sequence of SON objects."},
     {NULL, NULL, 0, NULL}
 };
 
@@ -1180,6 +1389,7 @@ PyMODINIT_FUNC init_cbson(void) {
     CBSONError = PyObject_GetAttrString(module, "InvalidDocument");
     InvalidName = PyObject_GetAttrString(module, "InvalidName");
     InvalidDocument = PyObject_GetAttrString(module, "InvalidDocument");
+    InvalidStringData = PyObject_GetAttrString(module, "InvalidStringData");
     Py_DECREF(module);
 
     module = PyImport_ImportModule("pymongo.son");
